@@ -70,7 +70,11 @@ class FreeIpaRpcAuthBackend(ModelBackend):
                 # failover server once and return user
                 logger.critical("Main FreeIPA server connection error")
                 if tries == 1 and self.settings.FAILOVER_SERVER:
-                    return self.authenticate(username, password, tries=tries+1)
+                    return self.authenticate(
+                        username=username,
+                        password=password,
+                        tries=tries+1
+                    )
                 else:
                     raise
 
@@ -82,7 +86,7 @@ class FreeIpaRpcAuthBackend(ModelBackend):
         :return:
         """
         groups = user_session.groups
-        groups += user_session.user_data['memberofindirect_group']
+        groups += user_session.user_data.get('memberofindirect_group', [])
         return list(set(groups))
 
     def update_user(self, user_session):
@@ -113,10 +117,11 @@ class FreeIpaRpcAuthBackend(ModelBackend):
         return user
 
     def update_user_attrs(self, user, user_session_data):
-
         for attr, key in self.settings.USER_ATTRS_MAP.items():
             attr_value = user_session_data[key]
-            setattr(user, attr, attr_value.pop() if isinstance(attr_value, list) else attr_value)  # noqa: E501
+            if isinstance(attr_value, list):
+                attr_value = attr_value.pop()
+            setattr(user, attr, attr_value)
 
     def update_user_groups(self, user, groups):
         """
@@ -127,9 +132,8 @@ class FreeIpaRpcAuthBackend(ModelBackend):
 
         # Update user groups
         if self.settings.UPDATE_USER_GROUPS:
+            user.groups.clear()
             user.groups.add(*Group.objects.filter(name__in=groups))
-            # Remove now invalid user groups if any
-            user.groups.remove(*user.groups.exclude(name__in=groups))
 
 
 class FreeIpaAuthSettings(object):
@@ -140,7 +144,11 @@ class FreeIpaAuthSettings(object):
         'FAILOVER_SERVER': None,
         'SSL_VERIFY': True,
         'UPDATE_USER_GROUPS': False,
-        'USER_ATTRS_MAP': {'first_name': 'givenname', 'last_name': 'sn', 'email': 'mail'},
+        'USER_ATTRS_MAP': {
+            'first_name': 'givenname',
+            'last_name': 'sn',
+            'email': 'mail'
+        },
         'ALWAYS_UPDATE_USER': True,
     }
 
@@ -154,3 +162,7 @@ class FreeIpaAuthSettings(object):
         for name, default in self.defaults.items():
             value = getattr(settings, prefix + name, default)
             setattr(self, name, value)
+        if getattr(self, 'FAILOVER_SERVER') is None:
+            logger.warning(
+                "FreeIPA Failover Server is not set. Proceed with caution."
+            )
